module SHABAL_INTERFACE(
	clk, rst_n,
	init, EOM, load, fetch, idata,                                // Input from Control FPGA
	ack, odata,                                                   // Output for Control FPGA
	busy, hash0, hash1, hash2, hash3, hash4, hash5, hash6, hash7, // Input from Shabal_CORE
	init_r, EOM_r, EN, idata32                                    // Output for Shabal_CORE
);  

//{{{ Input/Output
// From/For Control FPGA
input         clk, rst_n;
input         init, EOM, load, fetch;
input  [15:0] idata;
output        ack;
output [15:0] odata;

// From/For Shabal_CORE
input         busy;
input  [31:0] hash0, hash1, hash2, hash3, hash4, hash5, hash6, hash7;
output        init_r, EOM_r, EN;
output [31:0] idata32;
//}}}

//{{{ Registers and Wires
// Init Signal
reg        init_r;

// Load/Fetch Signal
reg        load_r, fetch_r;

// Ack Signal
reg        ack_r;

// End of MessageSignal
reg        EOM_r;

// Message/HV Register
reg [31:0] idata32;
reg [15:0] odata_r;

// Data Counter
reg [4:0]  data_count;

// Finite State Machine
reg [2:0]  state;
reg [2:0]  next_state;

// Enable Signal
wire EN;
//}}}

// Enable Signal
assign EN = ((state == 3'b010) && (data_count[0] == 1'b0))? 1'b1 : 1'b0; // when hash execution state and input data_count is odd

//{{{ Init/EOM
// Init Signal
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		init_r <= 1'b0;
	end
	else begin
		if (init) begin
			init_r <= 1'b1;
		end
		else begin
			init_r <= 1'b0;
		end
	end
end

always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		EOM_r <= 1'b0;
	end
	else begin
		if (EOM) begin
			if (state == 3'b000) begin
				EOM_r <= 1'b1;
			end
			else begin
				EOM_r <= EOM_r;
			end
		end
		else begin
			EOM_r <= EOM_r;
		end
	end
end
//}}}

//{{{ Load/Fetch Signal
// Load Signal
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		load_r <= 1'b0;
	end
	else begin
		if (load) begin
			load_r <= 1'b1;
		end
		else begin
			load_r <= 1'b0;
		end
	end
end

// Fetch Signal
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		fetch_r <= 1'b0;
	end
	else begin
		if (fetch) begin
			fetch_r <= 1'b1;
		end
		else begin
			fetch_r <= 1'b0;
		end
	end
end
//}}}

//{{{ Ack Signal
assign ack = ack_r;
always @(posedge clk or negedge rst_n)begin
	if (~rst_n) begin
		ack_r <= 1'b0;
	end
	else begin
		if (state == 3'b001 || state == 3'b100) begin // LOAD state (3'b001) or FETCH state (3'b100)
			ack_r <= 1'b1;
		end
		else begin
			ack_r <= 1'b0;
		end
	end
end
//}}}

//{{{// Output Register
assign odata = odata_r;
always @(posedge clk or negedge rst_n)begin
	if (~rst_n) begin
		odata_r <= 16'h0;
	end
	else begin
		if (state == 3'b011) begin
			if (data_count == 4'd0) begin
				odata_r <= {hash0[7:0],hash0[15:8]};
			end
			else if (data_count == 4'd1) begin
				odata_r <= {hash0[23:16],hash0[31:24]};
			end
			else if (data_count == 4'd2) begin
				odata_r <= {hash1[7:0],hash1[15:8]};
			end
			else if (data_count == 4'd3) begin
				odata_r <= {hash1[23:16],hash1[31:24]};
			end
			else if (data_count == 4'd4) begin
				odata_r <= {hash2[7:0],hash2[15:8]};
			end
			else if (data_count == 4'd5) begin
				odata_r <= {hash2[23:16],hash2[31:24]};
			end
			else if (data_count == 4'd6) begin
				odata_r <= {hash3[7:0],hash3[15:8]};
			end
			else if (data_count == 4'd7) begin
				odata_r <= {hash3[23:16],hash3[31:24]};
			end
			else if (data_count == 4'd8) begin
				odata_r <= {hash4[7:0],hash4[15:8]};
			end
			else if (data_count == 4'd9) begin
				odata_r <= {hash4[23:16],hash4[31:24]};
			end
			else if (data_count == 4'd10) begin
				odata_r <= {hash5[7:0],hash5[15:8]};
			end
			else if (data_count == 4'd11) begin
				odata_r <= {hash5[23:16],hash5[31:24]};
			end
			else if (data_count == 4'd12) begin
				odata_r <= {hash6[7:0],hash6[15:8]};
			end
			else if (data_count == 4'd13) begin
				odata_r <= {hash6[23:16],hash6[31:24]};
			end
			else if (data_count == 4'd14) begin
				odata_r <= {hash7[7:0],hash7[15:8]};
			end
			else if (data_count == 4'd15) begin
				odata_r <= {hash7[23:16],hash7[31:24]};
			end
		end
		else begin
			odata_r <= odata_r;
		end
	end
end
//}}}

//{{{ Data Counter
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		data_count <= 5'h0;
	end
	else begin
		if (state == 3'b001) begin // LOAD state
			if (data_count == 5'd31) begin
				if (~busy) begin
					data_count <= 5'h0;
				end
				else begin
					data_count <= data_count;
				end
			end
			else begin
				data_count <= data_count + 1'd1;
			end
		end
		else if (state == 3'b011) begin // FETCH state
			if (data_count == 5'd15) begin
				data_count <= 5'h0;
			end
			else begin
				data_count <= data_count + 1'd1;
			end
		end
		else begin
			data_count <= data_count;
		end
	end
end
//}}}

//{{{ Input Register
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		idata32 <= 32'h0;
	end
	else begin
		if (state == 3'b001) begin // LOAD state
			idata32 <= {idata[7:0],idata[15:8],idata32[31:16]};
		end
		else begin
			idata32 <= idata32;
		end
	end
end
//}}}

//{{{ Finite State Machine
always @(posedge clk or negedge rst_n) begin
	if (~rst_n) begin
		state <= 3'b000;
	end
	else begin
		state <= next_state;
	end
end

always @(load_r or fetch_r or EN or busy or state) begin
	case (state)
	3'b000 : begin /* neutral state */
		if (load_r) begin
			next_state <= 3'b001;
		end
		else if (fetch_r) begin
			next_state <= 3'b011;
		end
		else begin
			next_state <= 3'b000;
		end
	end
	3'b001 : begin /* load state */
		next_state <= 3'b010;
	end
	3'b010 : begin /* execution state */
		if (busy) begin
			next_state <= 3'b010;
		end
		else begin
			next_state <= 3'b000;
		end
	end
	3'b011 : begin /* fetch state */
		next_state <= 3'b100;
	end
	3'b100 : begin /* output state */
		next_state <= 3'b000;
	end
	default : begin
		next_state <= 3'b000;
	end
        endcase
end

endmodule
//}}}

// // vim:fdm=marker
